;###############################################################################
/* Grid for LilBuilder
06-02-2007(ahklerner)-Added Grid_Draw()

*/
;###############################################################################

;Returns/increases/decreases grid size
GRID(Action=""){
  static GRID := 0, GRID_Increment := 5              ;initial grid size and increments
  If !Action                                         ;return grid size when nothing is specified
      Return GRID

  If Action = Up                                     ;increase GRID on "UP"
    GRID += GRID_Increment
  Else If Action = Down                              ;increase GRID on "Down"
    GRID -= GRID_Increment
  Else If Action is integer
    GRID = %Action%
  Else                                               ;issue error otherwise
    MsgBox, Wrong Action in Grid()

  GRID := GRID < 0 ? 0 : GRID                        ;make sure it is >= 0
  ToolTip("Grid = " GRID, 2000, "Continue")          ;give feedback
  Return GRID                                        ;return new GRID value
}

;change grid size
;called when
; - hotkeys to change grid size are pressed 
Grid_Change:
  InStr(A_ThisHotkey, "Shift") ? GRID("Up") : GRID("Down")   ;increase/decrease pending on hotkey
;   If Grid_MoveCtrl2Grid                  ;??? option doesn't exist yet
;     SetTimer, Grid_MoveCtrl2Grid, -500
;   Else
    Grid_Draw()                            ;update grid
Return

; Grid_MoveCtrl2Grid:
;   Grid_MoveCtrl2Grid(Selector_vCtrlHWND())
;   Selector_Set2Ctrl()
;   Grid_Draw()
; Return
;
; Grid_MoveCtrl2Grid(HWND){
;     ControlGetPos, X, Y, W, H, , ahk_id %HWND%
;     GRID := GRID()
;     dx := Mod(X, GRID),dy := Mod(Y, GRID), dw := Mod(w, GRID), dh := Mod(h, GRID)
;     dx -= (dx ? GRID : 0), dy -= (dy ? GRID : 0), X -= dx, Y -= dy
;     w -= (w > dw ? dw : 0), h -= (h > dh ? dh : 0)
;     ControlMove,,x,y,w,h, ahk_id %HWND%
;   }

;draw the grid
Grid_Draw($Color="0xC0C0C0",$Weight=1) {
  Static Grid_Old$Size, cnt = 0
  
  GuiHWND := Selector_vGuiHWND()

  If !($Size := GRID()){                       ;return, when no grid is used
      If Grid_Old$Size {                            ;redraw gui, when it has been grid before
         WinSet, Redraw,, ahk_id %GuiHWND%
         Grid_Old$Size = 0
      }
      Return
  }

  ; Get rid of all of the old lines
  If (Grid_Old$Size != $Size) {
    WinSet, Redraw,, ahk_id %GuiHWND%
    Sleep, 50      ; Wait a bit...otherwise some lines are not drawn
  }
  Grid_Old$Size := $Size

  ;Get the position of the builder window
  WinGetPos, Grid_Line_x, Grid_Line_y, Grid_Window_h, Grid_Window_w, ahk_id %GuiHWND%

  ;figure out how many lines to draw
  Grid_Line_Count_W := Grid_Window_w / $Size, Grid_Line_Count_H := Grid_Window_h / $Size

  ; get the device context of the builder window
  Grid_hDC := DllCall("GetDC", UInt, GuiHWND)

  ; Create a pen to draw with
  Grid_hCurrPen := DllCall("CreatePen", UInt, 0, UInt, $Weight, UInt, $Color)
  DllCall("SelectObject", UInt,Grid_hDC, UInt,Grid_hCurrPen)


  ; Draw the grid lines
  Grid_Line_x = 0
  Loop, % Grid_Line_Count_H
  {
    ; increase the x position by the Grid size
    Grid_Line_x += $Size
    ; Set the begining of the line
    DllCall("gdi32.dll\MoveToEx", UInt, Grid_hDC, Uint,Grid_Line_x, Uint, 0, Uint, 0 )
    ; Set the end of the line
    DllCall("gdi32.dll\LineTo", UInt, Grid_hDC, Uint, Grid_Line_x, Uint, Grid_Window_w)
  }
  
  Grid_Line_y = 0
  Loop, % Grid_Line_Count_W
  {
    ; increase the y position by the Grid size
    Grid_Line_y += $Size
    ; Set the beginning of the line
    DllCall("gdi32.dll\MoveToEx", UInt, Grid_hDC, Uint,0, Uint, Grid_Line_y, Uint, 0 )
    ; Set the end of the line
    DllCall("gdi32.dll\LineTo", UInt, Grid_hDC, Uint, Grid_Window_h, Uint, Grid_Line_y)
  }

  ; Cleanup
  DllCall("ReleaseDC", UInt, 0, UInt, Grid_hDC)  ; Clean-up.
  DllCall("DeleteObject", UInt,Grid_hCurrPen)
}
