;###############################################################################
/* Gui for LilBuilder

*/
;###############################################################################

;create a user gui
Gui_Create() {
  local GuiHWND, WindowTitle         ;global context
  static GuiCnt = 9     ;start user guis with id 10, which leave 9 guis for the builder

  GuiCnt++                                               ;get GuiID
  Gui, %GuiCnt%:+LastFound +Resize +LabelGui_            ;set defaults
  GuiHWND := WinExist() + 0                              ;get its HWND
  Gui%GuiCnt%_HWND = %GuiHWND%                           ;HWND as global var, for Gui_Size
  GroupAdd, GroupOfUserGuis, ahk_id %GuiHWND%            ;add it to group for hotkeys
;   WinSet, Trans, 255
  WindowTitle = %GuiCnt%: %ScriptName% - r%SVN_RevNum% %SVN_RevDate%
  Gui, %GuiCnt%:Show, w550 h610, %WindowTitle%           ;create gui
;   Gui, %GuiCnt%:Add,Picture, x0 y0 vGrid%GuiHWND%, %A_ScriptDir%\res\grid.gif
  Gui(GuiHWND,"Create")                                  ;create container
  Gui(GuiHWND,"Resize", 1)                               ;set gui properties
  Gui(GuiHWND,"Toolwindow", 0)
  Gui(GuiHWND,"GuiID", GuiCnt)
  Gui(GuiHWND,"Content", WindowTitle)
  Project_AddUserGUI(GuiHWND)                            ;add to project
  Selector_vGuiHWND(GuiHWND)                             ;set as selected
  Selector_vCtrlHWND("No")                              ;clear selection
  Selector_Set2PosSize(-10,-10)                          ;hide selector
  ToolTip_ShowTips4Pos("Hide")
}

;gui closed, exit on close
Gui_Close:                                               
  If Project("DockWindows")
    ExitApp
  Else {
    Selector_Set2PosSize(-10, -10)                   ;hide selector
    Selector_vCtrlHWND("No")                         ;clear selection
    Gui, %A_Gui%:Hide
  }
return

;gui resizes, update grid
Gui_Size:
  Gui_Size(Gui%A_Gui%_HWND)
/* ??? eliminate these
*/
  %A_Gui%GuiWidth =  %A_GuiWidth%
  %A_Gui%GuiHeight = %A_GuiHeight%
Return
Gui_Size(GuiHWND){
  Grid_Draw()
  If (AnchoredCtrls := Gui(GuiHWND, "AnchoredCtrls"))
    Loop, Parse, AnchoredCtrls, |
      Menu_Anchor( GuiHWND, A_LoopField,  Control(A_LoopField, "AnchorValue"))
  Selector_Set2Ctrl()
  SetTimer, Gui_SizeUpdateSelector, 10
}
Gui_SizeUpdateSelector:
  If !GetKeyState("LButton"){
    SetTimer, Gui_SizeUpdateSelector, Off
    Selector_Set2Ctrl()
  }
Return

Gui_GetGuiHWNDwGuiID(GuiID){ ;???not used, easier with global var Gui<GuiID>_HWND
  ListOfGuiHWND := Project("ListOfGuiHWND")
  Loop, Parse, ListOfGuiHWND, |
      If (GuiID := Gui(A_LoopField, "GuiID"))
          Return A_LoopField
}

;refresh gui area, e.g. when controls got moved
Gui_Refresh:
  DllCall("InvalidateRect", "uint", Selector_vGuiHWND(), "uint", 0, "uint", 0)
  DllCall("UpdateWindow", "uint", Selector_vGuiHWND())
  Grid_Draw()
return

;add a control to the list of controls on a gui
Gui_AddControl(GuiHWND,CtrlHWND){
  Control(CtrlHWND,"BelongsTo", GuiHWND)
  OrderOfControls := Gui(GuiHWND,"OrderOfControls") . "|" . CtrlHWND
  If InStr(OrderOfControls, "|") = 1                 ;remove first | if any
    StringTrimLeft, OrderOfControls, OrderOfControls, 1
  Gui(GuiHWND,"OrderOfControls",OrderOfControls)
}

;remove a control from the list of controls on a gui OR tab page
Gui_RemoveControl(CtrlHWND){
  HWND := Control(CtrlHWND,"BelongsTo")  ;check to which item it belongs (gui or tab page)
  CtrlHWND = |%CtrlHWND%|
  If InStr(HWND, "T") {                           ;it's a tab page control
    OrderOfControls := "|" Control(HWND,"OrderOfControls") "|"     ;remove it from tab page control list
    StringReplace, OrderOfControls, OrderOfControls, %CtrlHWND%, |
    StringTrimRight, OrderOfControls, OrderOfControls, 1
    StringTrimLeft, OrderOfControls, OrderOfControls, 1
    Control(HWND,"OrderOfControls",OrderOfControls, OrderOfControls ? "" : "Clear")
  }
  Else {
    OrderOfControls := "|" Gui(HWND,"OrderOfControls") "|"         ;remove it from gui control list
    StringReplace, OrderOfControls, OrderOfControls, %CtrlHWND%, |
    StringTrimRight, OrderOfControls, OrderOfControls, 1
    StringTrimLeft, OrderOfControls, OrderOfControls, 1
    Gui(HWND,"OrderOfControls",OrderOfControls, OrderOfControls ? "" : "Clear")
  }
}

;return a list of all controls HWND, including ALL tab page controls
Gui_GetAllCtrlHWND(GuiHWND){
  OrderOfControls := Gui(GuiHWND,"OrderOfControls")
  Loop, Parse, OrderOfControls, |
    {
      AllControlsOnGui .= "|" A_LoopField
      If ("Tab" = Control(A_LoopField,"Type"))
          If (AllControlsOnTab := Tab_GetAllCtrlHWND(A_LoopField))
              AllControlsOnGui .= "|" AllControlsOnTab
    }
  Return SubStr(AllControlsOnGui,2)       ;return without leading pipe
}

;return a list of all control HWND, excluding controls on inactive tab pages
Gui_GetAllCtrlHWNDWTabActive(GuiHWND){
  OrderOfControls := Gui(GuiHWND,"OrderOfControls")
  Loop, Parse, OrderOfControls, |
    {
      AllControlsOnGui .= "|" A_LoopField
      If ("Tab" = Control(A_LoopField,"Type"))
          If (AllControlsOnTab := Tab_GetAllCtrlHWNDFromActivePages(GuiHWND, A_LoopField))
              AllControlsOnGui .= "|" AllControlsOnTab
    }
  Return SubStr(AllControlsOnGui,2)       ;return without leading pipe
}

;dictionary to store all gui relevant properties.
Gui(HWND,PropertyNameOrAction,Value="",Clear=""){
  Static MasterDic, init = 0

  If !init {  ;set up a master dic once to link HWNDs to their pDic
    init++
    MasterDic := Dic("insensitive")
  }

  If (HWND = "Remove" And PropertyNameOrAction = "All"){ ;delete all dics
    Loop{                                                ;delete all Gui dics
      If (HWND := Dic_NextKey(penum, MasterDic))
        Dic("delete", Dic_Get(MasterDic, HWND))
      If !penum
        Break
    }
    Dic("delete",MasterDic)                              ;delete master dic
    init = 0                                             ;reset init state
  }

  Else If (PropertyNameOrAction = "Remove"){   ;remove dic for Gui
    Dic("delete", Dic_Get(MasterDic, HWND))    ;delete the dic for the Gui
    Dic_Remove(MasterDic, HWND)                ;remove dic from master dic
  }

  Else If (PropertyNameOrAction = "Create")    ;create a dic for Gui
    Dic_Set(MasterDic, HWND, Dic("insensitive"))    ;store pDic of dic in master list under the HWND key
   
  Else If (PropertyNameOrAction = "AllPropertyNames"){ ;return all Property Names for project
    Loop, % Dic_Count(Dic_Get(MasterDic, HWND))
        AllPropertyNames .= "`n" . Dic_NextKey(ForInternalUse, Dic_Get(MasterDic, HWND))
    StringTrimLeft, AllPropertyNames, AllPropertyNames, 1
    Sort, AllPropertyNames
    Return AllPropertyNames  
  }

  Else If Value is not space  ;assign value
    Dic_Set(Dic_Get(MasterDic, HWND), PropertyNameOrAction, Value)   ;get pDic from master dic and assign value
   
  Else If Clear is not space  ;clear value
    Dic_Set(Dic_Get(MasterDic, HWND), PropertyNameOrAction, "")      ;get pDic from master dic and clear value

  Else                        ;return value
    Return Dic_Get(Dic_Get(MasterDic, HWND), PropertyNameOrAction)   ;get pDic from master dic and return value
   
  Return 1
}
